#!/usr/bin/python
#
# Prints a simple list of stuff that is outdated on the system
#
# This file is a portion of the Red Hat Network Panel Applet
#
# Copyright (C) 1999-2002 Red Hat, Inc. All Rights Reserved.
# Distributed under GPL version 2.
#
# $Id: needed-packages.py,v 1.8 2002/04/03 23:59:03 cturner Exp $

import sys

sys.path.append("/usr/share/rhn/up2date_client")
original_arg_list = sys.argv[:]
sys.argv = [ sys.argv[0] ]

import clap
import rhn_utils
from rhn_applet_model import rhnAppletModelTUI

def main(arglist = []):
    arg_obj = clap.Clap([
        "help",
        ['v', 'verbose', None,     'Increase verbosity'],
        ['h', 'help',    None,     'Get this help screen'],
        ['u', 'url',     type(""), 'Change the default URL we talk to'],
        ])
    arg_obj.process(arglist)

    if arg_obj.getLong("verbose"):
        rhn_utils.set_log_level(1)
    # if a different XMLRPC server is requested...
    url = arg_obj.getLong("url")
    if not url:
        url = rhn_utils.get_config("server_url")
    uuid = rhn_utils.get_config("uuid")

    model = rhnAppletModelTUI(None, url, uuid)

    needed_packages, ignored_packages = model.needed_packages()
    needed_errata = {}
    
    if len(needed_packages):
        print
        print "The following is a list of outdated packages on your system:"
        print
        print "%-*s %-*s %-*s" % (40, "Name", 15, "Version", 20, "Release")
        print "-" * (40 + 15 + 20 + 2)
        
        for pkg in needed_packages:
            if pkg["errata_advisory"]:
                needed_errata[pkg["errata_advisory"]] = [ pkg["errata_advisory"],
                                                          pkg["errata_synopsis"] ]

            print "%(name)-40s %(version)-15s %(release)-20s" % pkg
        print
        # and now the errata list
        if needed_errata:
            print "%-*s %s\n%s" % (18, "Advisory", "Synopsis", "-" * 76)
            for errata in needed_errata.values():
                print "%-18s %s" % (errata[0], errata[1])
            print
        
    else:
        print "No packages needed."


if __name__ == "__main__":
    main(original_arg_list)
