# This file is a portion of the Red Hat Network Panel Applet
#
# Copyright (C) 1999-2002 Red Hat, Inc. All Rights Reserved.
# Distributed under GPL version 2.
#
# Author: Chip Turner
#
# $Id: rhn_applet_dialogs.py,v 1.18 2002/04/15 22:50:51 cturner Exp $

import gettext

GETTEXT_DOMAIN = "rhn-applet"
gettext.bindtextdomain (GETTEXT_DOMAIN, "/usr/share/locale")
gettext.textdomain (GETTEXT_DOMAIN)
_=gettext.gettext

import gnome.applet
import gnome.ui
import gnome
import gnome.uiconsts
import gnome.url

import gtk
import gtkhtml
import libglade

import os
class rhnGladeWindow:
    def __init__(self, filename, window_name):
        self.filename = filename
        if not os.access(self.filename, os.R_OK):
            self.filename = "/usr/share/rhn/rhn_applet/" + self.filename
            
        self.xml = libglade.GladeXML(self.filename, window_name, domain="rhn-applet")
        self.window = self.xml.get_widget(window_name)
        
    def get_widget(self, widget):
        return self.xml.get_widget(widget)

class rhnAppletNoticeWindow(rhnGladeWindow):
    def __init__(self, parent):
        rhnGladeWindow.__init__(self, "rhn_applet.glade", "notice_window_2")
        
        self.parent = parent
        self.window.signal_connect('delete_event', self.close_window)

        self.package_list = self.get_widget('update_clist')

        self.ignore_list = self.get_widget('ignore_clist')
        self.ignore_list_contents = []

        self.available_list = self.get_widget('available_clist')
        self.available_list_contents = []

        self.notebook = self.get_widget('notice_notebook')
        self.critical_tab = None
        self.critical_tab_contents = None
        
        self.xml.signal_autoconnect (
            {
            "on_launch_up2date_clicked" : self.on_up2date,
            "on_ignore_clicked" : self.on_ignore_clicked,
            "on_unignore_clicked" : self.on_unignore_clicked,
            "on_close_clicked" : self.on_close,
            })
        
    def on_up2date(self, button):
        self.parent.launch_up2date()
        
    def on_close(self, close_button):
        self.close_window()

    def close_window(self, *rest):
        self.window.destroy()
        self.parent.notice_window_closed()

    def clear_window(self):
        self.package_list.clear()

        self.available_list.clear()
        self.available_list_contents = []

        self.ignore_list.clear()
        self.ignore_list_contents = []

    def on_link_clicked(self, html, url):
        print "url: %s" % url
        
    def set_critical(self, text, critical_active):
        if not self.critical_tab_contents:
            html = gtkhtml.GtkHTML()
            sw = gtk.GtkScrolledWindow()
            sw.set_policy(gtk.POLICY_AUTOMATIC, gtk.POLICY_AUTOMATIC)
            sw.set_border_width(2)
            sw.add(html)
            
            tab_label = gtk.GtkLabel(_("Critical Information"))
            tab_label.show()

            label_style = self.notebook.get_style()
            rgb = label_style.bg[gtk.STATE_NORMAL]
            html.set_default_background_color(rgb)

            html.show()
            sw.show()

            self.critical_tab = self.notebook.prepend_page(sw, tab_label)
            self.critical_tab_contents = sw

            if critical_active:
                self.notebook.set_page(self.notebook.page_num(self.critical_tab_contents))
            
            self.set_critical_tab_text(text)

            html.connect('link_clicked', self.on_link_clicked)
        else:
            if self.critical_tab_text != text:
                self.set_critical_tab_text(text)

    def set_critical_tab_text(self, text):
        self.critical_tab_text = text

        html = self.critical_tab_contents.children()[0]
        html.load_from_string(text)

    def remove_critical(self):
        if not self.critical_tab_contents:
            return

        self.notebook.remove_page(self.notebook.page_num(self.critical_tab_contents))

    def on_ignore_clicked(self, *data):
        if not self.available_list.selection:
            return
        
        i = self.available_list.selection[0]
        self.parent.set_ignored(self.available_list_contents[i], 1)
    
    def on_unignore_clicked(self, *data):
        if not self.ignore_list.selection:
            return

        i = self.ignore_list.selection[0]
        self.parent.set_ignored(self.ignore_list_contents[i], 0)

    def add_package(self, name, installed, avail):
        if self.parent.model.is_package_ignored(name):
            self.ignore_list_contents.append(name)
        else:
            self.available_list_contents.append(name)
            self.package_list.append((name, installed, avail))

    def redraw_lists(self):
        self.available_list.clear()

        self.available_list_contents.sort()
        for i in self.available_list_contents:
            self.available_list.append((i,))

        self.ignore_list.clear()

        self.ignore_list_contents.sort()
        for i in self.ignore_list_contents:
            self.ignore_list.append((i,))

        

class rhnRegistrationPromptDialog(rhnGladeWindow):
    def __init__(self, parent):
        rhnGladeWindow.__init__(self, "rhn_applet.glade", "need_to_register_dialog")
        
        self.parent = parent
        self.window.signal_connect('delete_event', self.close_dialog)
        self.xml.signal_autoconnect (
            {
            "on_launch_rhnreg_clicked" : self.on_rhnreg,
            "on_close_clicked" : self.on_close,
            })

    def raise_(self):
        self.window.get_window().raise_()

    def set_transient(self, papa):
        self.window.set_transient_for(papa.window)
        
    def on_rhnreg(self, button):
        self.parent.launch_rhnreg()
        self.close_dialog()
        
    def close_dialog(self, *rest):
        self.window.destroy()
        self.parent.rhnreg_dialog_closed()

    def on_close(self, close_button):
        self.close_dialog()



class rhnRPMReadErrorDialog(rhnGladeWindow):
    def __init__(self, parent):
        rhnGladeWindow.__init__(self, "rhn_applet.glade", "need_to_register_dialog")

        self.parent = parent
        self.window.signal_connect('delete_event', self.close_dialog)
        self.xml.signal_autoconnect (
            {
            "on_close_clicked" : self.on_close,
            })

    def raise_(self):
        self.window.get_window().raise_()

    def set_transient(self, papa):
        self.window.set_transient_for(papa.window)
        
    def close_dialog(self, *rest):
        self.window.destroy()
        self.parent.rpm_error_dialog_closed()

    def on_close(self, close_button):
        self.close_dialog()

class rhnAppletAboutWindow:
    def __init__(self, parent):
        self.window = gnome.ui.GnomeAbout("Red Hat Network Alert Notification Tool", "1.0.6", "Copyright (C) 2002, Red Hat, Inc.", [ "Red Hat Network Team", "rhn-feedback@redhat.com" ])
        self.window.set_modal(gtk.TRUE)
        self.window.signal_connect("destroy", self.on_close)
        self.parent = parent
        self.window.show()

    def close_dialog(self, *rest):
        self.parent.about_dialog_closed()

    def on_close(self, *data):
        self.close_dialog()

class rhnAppletFirstTimeDruid(rhnGladeWindow):
    def __init__(self, parent, proxy_url, proxy_username, proxy_password):
        rhnGladeWindow.__init__(self, "rhn_applet.glade", "first_time_druid")
        
        self.parent = parent
        self.window.signal_connect('delete_event', self.close_dialog)
        self.xml.signal_autoconnect (
            {
            "on_cancel" : self.on_cancel,
            "on_remove_from_panel" : self.on_remove_from_panel,
            "on_finish" : self.on_finish,
            })

        html_sw = self.get_widget("tos_window")
        self.tos_html = gtkhtml.GtkHTML()
        html_sw.add(self.tos_html)
        self.tos_html.show()
        html_sw.show()
        self.tos_html.connect('link_clicked', self.on_link_clicked)
        self.tos_html.load_from_string(_("""
Red Hat Network provides an intelligent, proactive management service
for your Red Hat Linux-based system.  Red Hat Network has the latest
Red Hat information, updates, and services to make your systems more
secure and reliable.  This application is designed to inform you when
updates are available for your system, but does not save any
personally identifiable information about you or your system to the
Red Hat Network unless you choose to subscribe to Red Hat Network.
Use of this applet by itself does not imply any agreement with Red Hat
Network.
<br><br>
Use of the up2date service, or use of this applet in conjunction with the
up2date service, is governed by the Red Hat Network Services Use and
Subscription Agreement, which may be reviewed at
<a href="https://rhn.redhat.com/help/terms.pxt">https://rhn.redhat.com/help/terms.pxt</a>.
<br><br>
Red Hat Network's privacy policy may be reviewed at 
<a href="https://rhn.redhat.com/help/security.pxt">https://rhn.redhat.com/help/security.pxt</a>.
<br><br>
If you do not wish to have this application appear on your panel,
click the 'Remove From Panel' button below.  Once removed, you can return it to your panel at any time by
choosing 'Red Hat Network Alert Notification Tool' from 'Add to Panel|Applet|Monitors' in the Panel menu.
        """))
                    
        
        self.enable_proxy = self.get_widget("enable_proxy_check")
        self.enable_proxy.signal_connect("toggled", self.on_enable_proxy_toggle)

        self.enable_auth = self.get_widget("use_auth_check")
        self.enable_auth.signal_connect("toggled", self.on_use_auth_toggle)

        self.proxy_entry = self.get_widget("proxy_entry")
        self.username_entry = self.get_widget("username_entry")
        self.password_entry = self.get_widget("password_entry")
        self.username_entry_label = self.get_widget("username_entry_label")
        self.password_entry_label = self.get_widget("password_entry_label")

        self.use_auth = 0
        self.use_proxy = 0

        self.proxy_entry.set_text(proxy_url)
        if proxy_url:
            self.use_proxy = 1
            self.enable_proxy.set_sensitive(gtk.TRUE)
            self.enable_proxy.activate()

            if proxy_username:
                self.use_auth = 1
                self.username_entry.set_text(proxy_username)
                self.password_entry.set_text(proxy_password)
                self.enable_auth.set_sensitive(gtk.TRUE)
                self.enable_auth.activate()

    def on_link_clicked(self, html, url):
        gnome.url.show(url)
        
    def on_enable_proxy_toggle(self, button):
        state = button.get_active()
        self.use_proxy = state

        self.get_widget("proxy_entry").set_sensitive(state)
        self.enable_auth.set_sensitive(state)

        if self.use_auth:
            self.username_entry.set_sensitive(state)
            self.username_entry_label.set_sensitive(state)
            self.password_entry.set_sensitive(state)
            self.password_entry_label.set_sensitive(state)
        
    def on_use_auth_toggle(self, button):
        state = button.get_active()
        self.use_auth = state

        self.username_entry.set_sensitive(state)
        self.username_entry_label.set_sensitive(state)
        self.password_entry.set_sensitive(state)
        self.password_entry_label.set_sensitive(state)
        
    def close_dialog(self, *data, **kwarg):
        self.window.destroy()
        if kwarg.has_key("remove"):
            self.parent.first_time_druid_closed(kwarg["remove"])
        else:
            self.parent.first_time_druid_closed(0)

    def on_cancel(self, cancel_button):
        self.close_dialog(remove=0)

    def on_remove_from_panel(self, cancel_button):
        self.close_dialog(remove=1)

    def on_finish(self, *data):
        if self.use_proxy:
            args = [ self.proxy_entry.get_text() ]
            if self.use_auth:
                args.append(self.username_entry.get_text())
                args.append(self.password_entry.get_text())

            apply(self.parent.set_proxy, args)
                                  
        else:
            self.parent.set_proxy()
            
        self.parent.user_consented()
        self.close_dialog()

class rhnAppletErrorDialog(rhnGladeWindow):
    def __init__(self, parent, error):
        self.window = gnome.ui.GnomeErrorDialog(str(error))
        self.window.set_modal(gtk.TRUE)
        self.window.signal_connect("destroy", self.on_close)
        self.parent = parent
        self.window.show()

    def close_dialog(self, *rest):
        self.parent.error_dialog_closed()

    def on_close(self, *data):
        self.close_dialog()
        
class rhnAppletExceptionDialog:
    def __init__ (self, parent, text):
        self.parent = parent
        win = gnome.ui.GnomeDialog ("Exception Occured")
        win.connect ("clicked", self.close_dialog)
        # win.append_button ("Mail Traceback to Red Hat")
        win.append_button_with_pixmap ("OK", gnome.ui.STOCK_BUTTON_OK)
        textbox = gtk.GtkText()
        textbox.insert_defaults (text)
        sw = gtk.GtkScrolledWindow ()
        sw.add (textbox)
        sw.set_policy (gtk.POLICY_AUTOMATIC, gtk.POLICY_AUTOMATIC)

        hbox = gtk.GtkHBox(gtk.FALSE)

        info = gtk.GtkLabel(_("An unhandled exception has occured.  This "
                              "is most likely a bug.  Please copy the "
                              "full text of this exception into an email "
                              "and send it to applet@rhn.redhat.com.  Thank you."))
        
        info.set_line_wrap (gtk.TRUE)
        info.set_usize (400, -1)

        hbox.pack_start (sw, gtk.TRUE)
        win.vbox.pack_start (info, gtk.FALSE)            
        win.vbox.pack_start (hbox, gtk.TRUE)
        win.set_usize (500, 300)
        win.set_position (gtk.WIN_POS_CENTER)
        win.show_all ()
        win.set_modal(gtk.TRUE)
        self.window = win

    def close_dialog(self, *rest):
        self.parent.error_dialog_closed()
        self.window.close()

    def on_close(self, *data):
        self.close_dialog()
