# This file is a portion of the Red Hat Network Panel Applet
#
# Copyright (C) 1999-2002 Red Hat, Inc. All Rights Reserved.
# Distributed under GPL version 2.
#
# $Id: rhn_utils.py,v 1.24 2002/04/12 07:57:17 cturner Exp $

import pwd
import os
import string
import sys
import shelve
import exceptions

import xmlrpclib

import gettext

from rhn_applet_rpm import rhnAppletRPM

GETTEXT_DOMAIN = "rhn-applet"
gettext.bindtextdomain (GETTEXT_DOMAIN, "/usr/share/locale")
gettext.textdomain (GETTEXT_DOMAIN)
_=gettext.gettext

# up2date_client/config.py
from config import Config

config_obj = None
user_config_obj = None

class rhnAppletConfig(Config):
    "derived from the Config class, with prepopulated default values"

    def __init__(self):
        Config.__init__(self)

        self.fileName = "/etc/sysconfig/rhn/rhn-applet"
        self.dict["use_ca_cert"] = (_("The location of the SSL CA certificate."), "/usr/share/rhn/RHNS-CA-CERT")

class rhnAppletUserConfig(Config):
    "derived from the Config class, with prepopulated default values"

    def __init__(self):
        Config.__init__(self)

        file = pwd.getpwuid(os.getuid())[5]
        file = file + "/.rhn-applet.conf"
        self.fileName = file
        self.dict["IgnoredPackages"] = (_("Package names to ignore when counting updates"), "")
        self.dict["Consent"] = (_("User consents to connection to RHN for updates"), "")
        self.dict['httpProxy'] = (_("HTTP proxy in host:port format, e.g. squid.redhat.com:3128"), "")
        self.dict['proxyUser'] = (_("The username for an authenticated proxy"), "")
        self.dict['proxyPassword'] = (_("The password to use for an authenticated proxy"), "")

def get_config(str):
    global config_obj
    
    if not config_obj:
        config_obj = rhnAppletConfig()
        config_obj.load()

    return config_obj.readEntry(str)

def get_user_config(str):
    global user_config_obj
    
    if not user_config_obj:
        user_config_obj = rhnAppletUserConfig()

        if not os.access(user_config_obj.fileName, os.F_OK):
            user_config_obj.save()
        
        user_config_obj.load()

    return user_config_obj.readEntry(str)

def set_user_config(str, val):
    global user_config_obj
    
    if not user_config_obj:
        user_config_obj = rhnAppletUserConfig()

        if not os.access(user_config_obj.fileName, os.F_OK):
            user_config_obj.save()
        
        user_config_obj.load()

    user_config_obj.writeEntry(str, val)
    user_config_obj.save()


DEBUG_LEVEL = 0
def set_log_level(new_level):
    global DEBUG_LEVEL
    DEBUG_LEVEL = new_level

def log_debug(str, level = 1):
    global DEBUG_LEVEL
    if DEBUG_LEVEL >= level:
        sys.stderr.write(str + "\n")

class rhnAppletException(exceptions.Exception):
    def __init__(self, args = None):
        self.args = args
    def __str__(self):
        return "Error: %s" % self.args

class rhnAppletNetworkException(rhnAppletException):
    def __init__(self, errno, errmsg):
        self.errno = errno
        self.errmsg = errmsg

    def __str__(self):
        return "Network Error %d: %s" % (self.errno, self.errmsg)

class rhnAppletRPCFault(rhnAppletException):
    def __init__(self, fault_number, fault_string):
        self.fault_number = fault_number
        self.fault_string = fault_string

    def __str__(self):
        return "Server Communication Error %d: %s" % (self.fault_number, self.fault_string)
    
